/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or 1any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer: Paul KEMLER
 * --------------------------------------------------------------------------
 * $Id: F_JonasAdminLink.java,v 1.1 2005/05/19 12:04:53 kemlerp Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.examples.clients.jonasadmin.navigation;


import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Vector;

import junit.framework.TestSuite;

import org.objectweb.jonas.examples.clients.jonasadmin.util.JProperties;
import org.objectweb.jonas.examples.clients.jonasadmin.util.JonasAdminAuth;
import org.objectweb.jonas.examples.util.JExampleTestCase;
import org.xml.sax.SAXException;

import com.meterware.httpunit.HttpException;
import com.meterware.httpunit.HttpNotFoundException;
import com.meterware.httpunit.HttpUnitOptions;
import com.meterware.httpunit.WebForm;
import com.meterware.httpunit.WebImage;
import com.meterware.httpunit.WebLink;
import com.meterware.httpunit.WebResponse;

/**
 * Define a class to test the JOnAS Admin console
 * Test all links
 * @author kemlerp
 *
 */
public class F_JonasAdminLink extends JExampleTestCase {

    /**
     * URL of the login page
     */
    private static final String URL_JONASADMIN = "/jonasAdmin/";

    /**
     * URL of the deployment of EARs
     */
    private static final String URL_JONASADMIN_DEPLOYEAR = "/jonasAdmin/EditDeploy.do?type=ear";

    /**
     * NAME of FRAME tree
     */
    private static final String FRAME_TREE = "tree";

    /**
     * NAME of FRAME content
     */
    private static final String FRAME_CONTENT = "content";

    /**
     * SUCCESS CODE
     */
    private static final int HTTP_SUCCESS_CODE = 200;

    /**
     * NAME OF THE EAR LINK
     */
    private static final String EAR_LINK_NAME = "Applications (EAR)";

    /**
     * Time of beginning testAllLinks
     */
    private String beginTime;

    /**
     * Time of ending testAllLinks
     */
    private String endTime;


    /**
     * Constructor with a specified name
     * @param s name
     */
    public F_JonasAdminLink(String s) {
        super(s, URL_JONASADMIN);
    }

    /**
     * Main method
     * @param args the arguments
     */
    public static void main(String[] args) {

        String testtorun = null;
        // Get args
        for (int argn = 0; argn < args.length; argn++) {
            String sArg = args[argn];
            if (sArg.equals("-n")) {
                testtorun = args[++argn];
            }
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new F_JonasAdminLink(testtorun));
        }
    }

    /**
     * Get a new TestSuite for this class
     * @return a new TestSuite for this class
     */
    public static TestSuite suite() {
        return new TestSuite(F_JonasAdminLink.class);
    }

    /**
     * Setup need for these tests
     * jonasAdmin is required
     * @throws Exception if it fails
     */
    protected void setUp() throws Exception {
        super.setUp();

        // Get time
        beginTime = getTime();

        useWar("jonasAdmin");
    }

    /**
     * Test all jonasAdmin links
     * @throws Exception if error occurs
     *
     */
    public void testAllLinks() throws Exception {

        Vector viewedURLs = new Vector();
        WebResponse wr;
        JProperties jProp = new JProperties();

        // Login to jonas admin
        try {
            JonasAdminAuth.doValidAuth(wc, url);
        } catch (Exception e) {
            fail("authentification failed : \n" + e);
        }

        // Disable errors of javascript
        HttpUnitOptions.setExceptionsThrownOnScriptError(false);
        // Disable exception thrown on error status
        HttpUnitOptions.setExceptionsThrownOnErrorStatus(false);

        // add links
        // deploy ear
        try {
            if (jProp.isEar()) {
                useEar("alarm");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        // Get the frame "tree"
        WebResponse treeFrame = wc.getFrameContents(FRAME_TREE);

        // Open tree
        treeFrame = openTree(treeFrame);

        // Verify all links are not viewTree.do
        verifyLinks(treeFrame, viewedURLs);
    }

    /**
     * Open tree
     * @param wr Tree Frame Web Response
     * @return Tree Frame Web Response with opened tree
     */
    public WebResponse openTree(WebResponse wr) {
        try {
            WebImage[] frameTreeImages = wr.getImages();
            for (int i = 0; i < frameTreeImages.length; i++) {
                if (frameTreeImages[i].getSource().indexOf("node_close") != -1) {
                   frameTreeImages[i].getLink().click();
                   // get frame tree response
                   wr = wc.getFrameContents(FRAME_TREE);
                   // update frameTreeImages
                   frameTreeImages = wr.getImages();
                }
            }
        } catch (SAXException e1) {
            e1.printStackTrace();
        } catch (IOException e) {
            fail("failed to open tree. " + e);
        }
        return wr;
    }

    /**
     * Verify all links in frame
     * @param frame The frame
     * @param viewedURLs All URLs have just been viewed
     */
    public void verifyLinks(WebResponse frame, Vector viewedURLs) {
        // Disable errors of javascript
        HttpUnitOptions.setExceptionsThrownOnScriptError(false);
        // Disable javascript
        HttpUnitOptions.setScriptingEnabled(false);
        // Disable exception thrown on error status
        HttpUnitOptions.setExceptionsThrownOnErrorStatus(false);

        WebLink[] links = null;

        try {
            // Get links
            links = frame.getLinks();
            WebResponse wr;

            if (frame.getFrameName().equals("tree")) {
                // Click links of the tree
                for (int i = 0; i < links.length; i++) {
                    // Verify all links are not viewTree.do and don't contain "://"
                    if (links[i].getURLString().indexOf("viewTree.do") == -1 && links[i].getURLString().indexOf("://") == -1) {
                        if (!viewedURLs.contains(links[i].getURLString())) {
                            try {
                                wr = links[i].click();
                                assertEquals("a code of 200 (success) should be returned. ", HTTP_SUCCESS_CODE, wr.getResponseCode());
                            } catch (HttpNotFoundException e) {
                                fail("failed to click :\nlink = " + links[i].asText() + ", URL = " + links[i].getURLString() + "\n" + e);
                            } catch (HttpException e) {
                                fail("failed to click :\nlink = " + links[i].asText() + ", URL = " + links[i].getURLString() + "\n" + e);
                            } catch (IOException e) {
                                fail("failed to click :\nlink = " + links[i].asText() + ", URL = " + links[i].getURLString() + "\n" + e);
                            }
                            // Add URL to viewed URLs
                            viewedURLs.add(links[i].getURLString());
                            // Verify content frame links
                            frame = wc.getFrameContents(FRAME_CONTENT);
                            verifyLinks(frame, viewedURLs);
                        }
                    }
                }
            } else {
                // Click links of not tree frames
                for (int i = 0; i < links.length; i++) {
                    // Verify all links are not viewTree.do
                    if (links[i].getURLString().indexOf("viewTree.do") == -1 && links[i].getURLString().indexOf("://") == -1) {
                        if (!viewedURLs.contains(links[i].getURLString())) {
                            try {
                                wr = links[i].click();
                                assertEquals("a code of 200 (success) should be returned. ", HTTP_SUCCESS_CODE, wr.getResponseCode());
                            } catch (HttpNotFoundException e) {
                                fail("failed to click :\nlink = " + links[i].asText() + ", URL = " + links[i].getURLString() + "\n" + e);
                            } catch (HttpException e) {
                                fail("failed to click :\nlink = " + links[i].asText() + ", URL = " + links[i].getURLString() + "\n" + e);
                            } catch (IOException e) {
                                fail("failed to click :\nlink = " + links[i].asText() + ", URL = " + links[i].getURLString() + "\n" + e);
                            }
                            // Add URL to viewed URLs
                            viewedURLs.add(links[i].getURLString());
                            // Verify content frame links
                            frame = wc.getFrameContents(FRAME_CONTENT);
                            verifyLinks(frame, viewedURLs);
                        }
                    }
                }
            }
        } catch (SAXException e) {
            e.printStackTrace();
        }
    }

    /**
     * Tear Down
     * cleanUp action
     */
    public void tearDown() {
        JProperties jProp = new JProperties();

        // Get time
        endTime = getTime();

        // Get created files
        Vector createdFiles = getCreatedFiles();

        // Delete created files: server.xml
        deleteCreatedFiles(createdFiles);

        // Undeploy alarm.ear
        try {
            if (jProp.isEar()) {
                undeployEar("alarm");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Get current time
     * @return A String Time with the format yyyy-MM-dd.HH-mm-ss
     */
    private String getTime() {
        Calendar cal = Calendar.getInstance();
        Date date = cal.getTime();
        SimpleDateFormat formatter;
        formatter = new SimpleDateFormat("yyyy-MM-dd.HH-mm-ss");
        return formatter.format(date);
    }

    /**
     * Get "server.xml" created files by the test
     * @return The table of created files
     */
    private Vector getCreatedFiles() {
        Vector createdFiles = new Vector();

        // Get JONAS_BASE path
        String jonasBase = System.getProperty("jonas.base");

        // Get JONAS_BASE/conf path
        String conf = jonasBase + File.separator + "conf";

        // Get JONAS_BASE/conf directory
        File directory = new File(conf);

        // Get JONAS_BASE conf files
        File[] files = directory.listFiles();

        // First name of file to compare
        String beginName = "server.xml." + beginTime;

        // End name of file to compare
        String endName = "server.xml." + endTime;

        // Get the created files by the test
        for (int i = 0; i < files.length; i++) {
            if (files[i].getName().compareTo(beginName) >= 0) {
                if (files[i].getName().compareTo(endName) <= 0) {
                    createdFiles.add(files[i]);
                }
            }
        }
        return createdFiles;
    }

    /**
     * Delete files
     * @param files A table of files to delete
     */
    private void deleteCreatedFiles(Vector files) {
        for (int i = 0; i < files.size(); i++) {
            // Delete file
            ((File) files.get(i)).delete();
        }
    }

    /**
     * Undeploy an EAR
     * @param name String of EAR Name
     * @throws Exception if error occurs
     */
    private void undeployEar(String name) throws Exception {
        String fileName = name + ".ear";

        // Disable errors of javascript
        HttpUnitOptions.setExceptionsThrownOnScriptError(false);
        // Disable exception thrown on error status
        HttpUnitOptions.setExceptionsThrownOnErrorStatus(false);

        // Get the frame "tree"
        WebResponse treeFrame = wc.getFrameContents(FRAME_TREE);

        WebResponse wr = wc.getResponse(getAbsoluteUrl(URL_JONASADMIN_DEPLOYEAR));
        WebForm[] webForms = wr.getForms();
        WebForm webForm = webForms[0];

        String params = webForm.getParameterValue("undeploy");
        WebForm.Scriptable script = webForm.getScriptableObject();

        // Disable errors of javascript
        HttpUnitOptions.setExceptionsThrownOnScriptError(false);

        if (params.length() == 0) {
            params += fileName;
        } else {
            params += "," + fileName;
        }

        script.setParameterValue("undeploy", params);

        WebResponse submitUndeploy = webForm.submit();

        webForms = submitUndeploy.getForms();
        webForm = webForms[0];

        WebResponse endResp = webForm.submit();
    }
}